package org.gnu.gtk;

import org.gnu.glib.Boxed;
import org.gnu.glib.Handle;

public class TreeRowReference extends Boxed{

	/**
	 * Creates a row reference based on path. This reference will keep pointing 
	 * to the node pointed to by path, so long as it exists. It listens to all
	 * signals emitted by model, and updates its path appropriately. If path isn't
	 * a valid path in model, an IllegalArgumentException is thrown.
	 * 
	 * @param model The model the reference will use 
	 * @param path Valid TreePath to the node to keep a reference to
	 */
	public TreeRowReference(TreeModel model, TreePath path) throws IllegalArgumentException{
		super(init(model, path));
	}
	
	private static Handle init(TreeModel model, TreePath path){
		Handle hndl = gtk_tree_row_reference_new(model.getHandle(), path.getHandle());
		if(hndl == null)
			throw new IllegalArgumentException("Invalid treepath");
		
		return hndl;
	}
	
	/**
	 * @return the model which reference is monitoring in order to appropriately modify the path.
	 */
	public TreeModel getModel(){
		return TreeModel.getTreeModel(gtk_tree_row_reference_get_model(this.getHandle()));
	}
	
	/**
	 * @return Returns a path that the row reference currently points to, or NULL if the path
	 * pointed to is no longer valid.
	 */
	public TreePath getPath(){
		return TreePath.getTreePath(gtk_tree_row_reference_get_path(this.getHandle()));
	}
	
	/**
	 * @return TRUE if the reference refers to a current valid path.
	 */
	public boolean isValid(){
		return gtk_tree_row_reference_valid(this.getHandle());
	}
	
	protected native static final Handle gtk_tree_row_reference_new(Handle model, Handle path);
	protected native static final Handle gtk_tree_row_reference_get_model(Handle reference);
	protected native static final Handle gtk_tree_row_reference_get_path(Handle reference);
	protected native static final boolean gtk_tree_row_reference_valid(Handle reference);
}
